/**
* This component read/write the app configurations
**/
var AppConfig = {
    hasLocalstorage: false,
    useAdvancedHttp: false, // A passer à true pour utiliser le plugin advanced-http
    init: function(cb) {
        cordova.plugin.http.clearCookies();
        if(window.localStorage !== undefined) {
            this.hasLocalstorage = true;
            // native storage is less volatile than local storage
            // we restore localstorage from nativestorage on startup
            NativeStorage.initWithSuiteName("howrse");
            NativeStorage.keys(function(keys) {
                AppConfig.restore(keys, cb);
            }, function(err) {
                debug(err);
                cb();
            });
        } else {
            AppCordova.onFailure();
        }
    },
    // recursively restore nativestorage keys
    restore: function(keys, cb) {
        debug("[CONFIG] Restore");
        if(keys.length > 0) {
            var key = keys.shift();
            NativeStorage.getItem(key,
                function(value) {
                    AppConfig.set(key, value);
                    AppConfig.restore(keys, cb);
                },
                function(err) {
                    debug(err);
                    AppConfig.restore(keys, cb);
                }
            );
        } else {
            cb();
        }
    },
    /// retrieve data
    get: function(key) {
        if(this.hasLocalstorage) {
            return window.localStorage.getItem(key);
        }
        return undefined;
    },
    /// set data
    set: function(key, value) {
        if(this.hasLocalstorage) {

            // Cas particulier de "secret"
            // On ne veut pas écraser le secret par un secret expiré (common.js:cbBeforeShow écrase mobileAjaxJson.js:onBeforeHandle)
            if(key == "secret") {
                var secretExpired = AppConfig.get("secretExpired");
                // On va écrase le secret avec une valeur expirée, on ne fait rien
                if(secretExpired == value && value != undefined) {
                    return false;
                } else {
                    // RAS - On fait expirer la valeur actuelle du secret
                    AppConfig.set("secretExpired", AppConfig.get("secret"));
                }
            }

            debug("[CONFIG] Set value <" + value + "> for key <" + key + ">");
            NativeStorage.setItem(key, value);
            return window.localStorage.setItem(key, value);
        }
        return false;
    },
    /// unset data
    unset: function(key) {
        if(this.hasLocalstorage) {
            var value = this.get(key);
            debug("[CONFIG] Unset key <" + key + "> which has value <" + value + ">");
            NativeStorage.remove(key);
            return window.localStorage.removeItem(key);
        }
        return false;
    },
    /// IN CASE OF EMERGENCY ONLY!!!
    clear: function() {
        if(this.hasLocalstorage) {
            NativeStorage.clear();
            return window.localStorage.clear();
        }
        return false;
    }
};

