/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var AppCordova = {

    rootUrl: null,
    hasNetwork: false,
    hasInitialized: false,
    busy: false,
    medias: [],
    fonts: [],
    firstPage: null,
    lastInteraction: null,

    // Application Constructor
    initialize: function() {
        this.bindEvents();
        this.rootUrl = window.location.href;
    },

    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        document.addEventListener('deviceready', AppCordova.onDeviceReady, false);
        document.addEventListener('pause', AppCordova.onPause, false);
        document.addEventListener('resume', AppCordova.onResume, false);
        document.addEventListener("offline", AppCordova.onNetworkOff, false);
        document.addEventListener("online", AppCordova.onNetworkOn, false);
    },
    // deviceready Event Handler
    onDeviceReady: function() {
       // Those functions are asynchronous
       // ensure that anything is loaded before continuing
       //name of app
       cordova.getAppVersion.getAppName(function (name) {
            AppCordova.name = name;
            //package of app
            cordova.getAppVersion.getPackageName(function (pkg) {
                AppCordova.package = pkg;
                //version of app
                cordova.getAppVersion.getVersionNumber(function (version) {
                    AppCordova.version = version;
                    //initialize config
                    AppConfig.init(function () {
                        //initialize loc
                        AppLocalization.init(function () {
                            //load version
                            debug("Running cordova-" + cordova.platformId + "@" + cordova.version);
                            AppCordova.handshake();
                        });
                    });
                });
            });
        });
    },
    /// Hold game
    onPause: function() {
        // not invoke onpause, delayed on resume most of the time :/
        debug("onPause");
    },
    /// Restart from background
    onResume: function() {
        // refresh app if asked or asleep for more than 20 min
        debug("onResume");
        if(AppCordova._wakeUp()) {
            // clear content before background
            if(navigator !== undefined &&
               navigator.notification !== undefined &&
               navigator.notification.dismissAll !== undefined)  {
                navigator.notification.dismissAll();
            }
            // show splashscreen
            navigator.splashscreen.show();
            // simply reload index.html
            window.location.href = AppCordova.rootUrl;
        }
    },
    /// Get sleepingtime
    _wakeUp: function() {
        // apps not fully initialise : resume
        if(AppCordova.hasInitialized === false) {
            return true;
        }
        // prevent apps to resume thank to busy parameters (inapppurchase)
        if(AppCordova.busy === true) {
            return false;
        }
        // all ok, check lastInteraction
        if(AppCordova.lastInteraction != null) {
            var sleepStart = AppCordova.lastInteraction;
            var sleepEnd = new Date();
            // refresh app if asleep for more than 20 min
            return (sleepEnd.getTime() - sleepStart.getTime() >= 20*60*1000);
        }
        return true;
    },
    /// Reset apps
    reset: function() {
        AppCordova.busy = false;
        AppCordova.lastInteraction = null;
        AppCordova.onResume();
    },

    /// When app is ready
    handshake: function() {

        if(BuildInfo.debug == true || BuildInfo.buildType === "debug") {
            debug("Mode : Debug");
        } else {
            debug("Mode : Release");
        }

        // Handle fresh install
        var launchCount = AppConfig.get("launchCount");
        (typeof launchCount == "undefined" || launchCount == null) ? launchCount = 1 : launchCount++;
        AppConfig.set("launchCount", parseInt(launchCount));

        // It's the first install
        launchCount == 1 ? this.setFreshInstall(true) : this.setFreshInstall(false);
        if(this.getFreshInstall() == 1) {
            AppConfig.unset('currentPage');
        }

        // communicate
        var urlToCall = AppLocalization.getUrl()+"/handshake";
        debug("urlToCall : " + urlToCall);

        if(AppConfig.useAdvancedHttp == true) {
            var parameters = {
                guid : this.getDeviceId(),
                version : this.getVersion(),
                secret : AppConfig.get("secret"),
                isFreshInstall : this.getFreshInstall(),
                sid : AppConfig.get("sid")
            };
            if(typeof AppConfig.get("sid") != "undefined" && AppConfig.get("sid") != "null") {
                parameters.sid = AppConfig.get("sid");
            }
        } else {
            var elements = new Array(
                "guid=" + this.getDeviceId(),
                "version=" + this.getVersion(),
                "secret=" + AppConfig.get("secret"),
                "isFreshInstall=" + this.getFreshInstall()
            );
            if(typeof AppConfig.get("sid") != "undefined" && AppConfig.get("sid") != "null") {
                elements.push("sid=" + AppConfig.get("sid"));
            }
            var parameters = elements.join("&");
        }

        AppRequest.call(
            urlToCall,
            parameters,
            function(success) {

                if(typeof success !== 'object') {
                    return;
                }

                if(AppConfig.useAdvancedHttp == true) {
                    var data = JSON.parse(success.data);
                } else {
                    var data = success;
                }

                // refresh secret&currentPage
                if(typeof data.deviceToken !== "undefined") {
                    AppConfig.set('secret', data.deviceToken);
                    if(typeof data.page !== 'undefined') {
                        AppCordova.firstPage = data.page;
                    }
                } else {
                    AppConfig.unset('secret');
                    AppConfig.unset('currentPage');
                }
                // session id
                if(typeof data.sid !== "undefined") {
                    AppConfig.set("sid", data.sid);
                } else {
                    AppConfig.unset("sid");
                }
                // is a fresh install ?
                if(typeof data.isFreshInstall !== "undefined") {
                    AppCordova.setFreshInstall(data.isFreshInstall);
                }

                // load medias
                AppCordova.loadMedias(
                    data.medias,
                    function() {
                        AppCordova.onSuccess(data);
                        // replace body
                        document.body.innerHTML = data.body;
                    },
                    function() {
                        AppCordova.onFailure({errorCode: 3, message: AppLocalization.getText('loadingErrorText')});
                        // replace body
                        document.body.innerHTML = data.body;
                    }
                );

            },
            function(error) {
                AppCordova.onFailure(error);
            }
        );
    },
    // load medias given by handshake recursively
    loadMedias: function(medias, success, failure) {
        if(medias.length > 0) {
            this.medias = medias;
            var media = medias.shift();
            var filename = media['name'];
            var url = media['url'];
            var type = media['type'];
            var version = media['version'];
            var overwrite = ((version != AppConfig.get(filename+type)) ? true : false);
            debug("AppCordova: loadMedia "+url);

            AppFile.download(
                url,
                filename+'.'+type,
                overwrite,
                function(e) {
                    // media updated, load next
                    AppConfig.set(filename+type, version);
                    // what to do depending on type
                    switch(type) {
                        case 'eot': case 'woff2': case 'woff': case 'ttf': case 'svg':
                            AppCordova.loadFont(filename, e, type, function() { AppCordova.loadMedias(medias, success, failure); }, failure);
                            break;
                        case 'css':
                            AppCordova.loadCss(filename, e, type, function() { AppCordova.loadMedias(medias, success, failure); }, failure);
                            break;
                        case 'js':
                            AppCordova.loadJs(filename, e, type, function() { AppCordova.loadMedias(medias, success, failure); }, failure);
                            break;
                        default:
                            // load next
                            AppCordova.loadMedias(medias, success, failure);
                    }
                },
                function(e) {
                    if(failure) failure();
                }
            );
        } else {
            if(success) success();
        }
    },
    loadFont: function(filename, e, type, success) {

        //TODO:Need improvement to use WOFF et WOFF2

        if(type !== 'woff2'){
            success();
            return;
        }
        e.file(function(f){
            var reader = new FileReader();
            reader.onerror = function() {
                AppCordova.onFailure({errorCode: 4, message: AppLocalization.getText('loadingErrorText')});
            }
            reader.onloadend = function(ev){
                var str = ev.target.result;
                var tag = document.createElement("style");
                tag.innerHTML = "@font-face {"
                    +"font-family: '"+filename+"';"
                    +"src: url(data:application/x-font-"+type+";"+str+");"
                    +"font-weight: normal;"
                    +"font-style: normal;"
                    +"}";
                tag.onload = function() {
                    debug("AppCordova: loadFont "+filename);
                    AppCordova.fonts.push(filename);
                    success();
                }
                document.head.appendChild(tag);
            };
            reader.readAsDataURL(f);
        });
    },
    loadCss: function(filename, e, type, success) {
        e.file(function(f){
            var reader = new FileReader();
            reader.onerror = function() {
                AppCordova.onFailure({errorCode: 4, message: AppLocalization.getText('loadingErrorText')});
            }
            reader.onloadend = function(ev){
                var tag = document.createElement("style");
                tag.innerHTML = ev.target.result;
                tag.onload = function() {
                    debug("AppCordova: loadCss "+filename);
                    success();
                }
                document.head.appendChild(tag);
            };
            reader.readAsText(f);
        });
    },
    loadJs: function(filename, e, type, success) {
        // handle javascript errors of loaded file
        var onerror = function(message, url, line, col, errorObj) {
            debug("AppCordova: loadJs error "+filename);
            debug("Error : " + JSON.stringify(errorObj, null, 4));
            debug("Error message : " + JSON.stringify(message, null, 4));
            AppCordova.loadMedias = function() { /*dont load further medias*/ }
            AppCordova.onFailure({errorCode: 6, message: AppLocalization.getText('loadingErrorText')});
        }
        window.addEventListener("error", onerror);
        e.file(function(f){
            var reader = new FileReader();
            reader.onerror = function() {
                AppCordova.onFailure({errorCode: 5, message: AppLocalization.getText('loadingErrorText')});
            }
            reader.onloadend = function(ev){
                var tag = document.createElement("script");
                tag.type = "text/javascript";
                tag.charset = "utf-8";
                tag.text = [ev.target.result];
                document.head.appendChild(tag);
                debug("AppCordova: loadJs "+filename);
                window.removeEventListener("error", onerror);
                success();
            };
            reader.readAsText(f);
        });
    },

    /// When network sleep/wakeup
    onNetworkOn: function() {
        if(AppCordova.hasNetwork === false && AppCordova.hasInitialized === true) {
            window.plugins.toast.hide();
        }
        this.hasNetwork = true;
    },
    onNetworkOff: function() {
        if(AppCordova.hasNetwork === true && AppCordova.hasInitialized === true) {
            AppInterface.error(AppLocalization.getText('networkLostText'));
        }
        this.hasNetwork = false;
    },
    checkNetwork: function() {
        var networkState = navigator.connection.type;
        var states = {};
        states[Connection.UNKNOWN]  = 'Unknown connection';
        states[Connection.ETHERNET] = 'Ethernet connection';
        states[Connection.WIFI]     = 'WiFi connection';
        states[Connection.CELL_2G]  = 'Cell 2G connection';
        states[Connection.CELL_3G]  = 'Cell 3G connection';
        states[Connection.CELL_4G]  = 'Cell 4G connection';
        states[Connection.CELL]     = 'Cell generic connection';
        states[Connection.NONE]     = 'No network connection';
        debug('Connection type: ' + states[networkState]);
        if(networkState == Connection.NONE){
            this.onNetworkOff();
            return false;
        } else {
            this.onNetworkOn();
            return true;
        }
    },

    /// When initialisation failed
    onFailure: function(data) {
        // initialisation failed
        this.hasInitialized = false;
        // hide the native splashscreen
        navigator.splashscreen.hide();
        // if Navigation is defined: hide the custom splashscreen
        if(typeof Navigation !== "undefined" && typeof Navigation.hideSplashscreen !== "undefined") {
            Navigation.hideSplashscreen();
        }
        // title
        if(data !== undefined && data.title !== undefined) {
            var title = data.title;
        } else if(data !== undefined && data.errorCode !== undefined) {
            var title = AppLocalization.getText('errorTitle')+" #"+data.errorCode;
        } else {
            var title = AppLocalization.getText('internalTitle');
        }
        // message
        if(data !== undefined && data.message !== undefined) {
            var message = data.message;
        } else {
            var message = AppLocalization.getText('internalText');
        }
        // callback
        if(data !== undefined && data.callback !== undefined) {
            var callback = data.callback;
        } else {
            var callback = function() { AppCordova.onResume(); };
        }
        // button
        if(data !== undefined && data.button !== undefined) {
            var button = data.button;
        } else {
            var button = AppLocalization.getText('retryButton');
        }
        // warning
        AppInterface.alert(
            message,
            callback,
            title,
            button
        );
    },
    onSuccess: function(data) {
        // initialisation suceeded
        this.hasInitialized = true;
        // hide the native splashscreen
        navigator.splashscreen.hide();
        // if Navigation is defined: hide the custom splashscreen
        if(typeof Navigation !== "undefined" && typeof Navigation.hideSplashscreen !== "undefined") {
            Navigation.hideSplashscreen();
        }
    },

    /// Device properties
    getDeviceModel: function() {
        return device.model;
    },
    getDevicePlatform: function() {
        return device.platform;
    },
    getDeviceId: function() {
        return device.uuid;
    },
    getDeviceVersion: function() {
        return  device.version;
    },
    getDeviceManufacturer: function() {
        return device.manufacturer;
    },
    getDeviceVirtual: function() {
        return device.isVirtual;
    },
    getDeviceSerial: function() {
        return device.serial;
    },
    /// App properties
    getName: function() {
        return AppCordova.name;
    },
    getPackage: function() {
        return AppCordova.package;
    },
    getVersion: function() {
        return AppCordova.version;
    },
    getFreshInstall: function() {
        return AppConfig.get(AppLocalization.getLang() + "-isFreshInstall");
    },
    setFreshInstall: function(isFreshInstall) {
        if(isFreshInstall == true) {
            AppConfig.set(AppLocalization.getLang() + "-isFreshInstall", 1);
        } else {
            AppConfig.set(AppLocalization.getLang() + "-isFreshInstall", 0);
        }
    }

};

/**
* This function for debug purpose
*/
function debug(datas, forceDisplay) {

    forceDisplay = forceDisplay != "undefined" ? forceDisplay : false;

    if(BuildInfo.buildType === "debug") {
        var message = '';
        if(typeof datas === 'object') {
            message = "[DEBUG] " + JSON.stringify(datas);
        } else {
            message = "[DEBUG] " + datas;
        }

        console.log(message);

        if(forceDisplay == true) {
            alert(message);
        }
    }
}

/**
* Allow to open apps with a single link howrse://
*/
function handleOpenURL(url) {
    setTimeout(function() {
        var currentPage = url.replace("howrse:/", '').replace(/^(?:\/\/|[^/]+)*\//, '');
        if(currentPage !== '' && AppCordova.rootUrl != currentPage) {
            AppConfig.set('currentPage', currentPage);
        }
    }, 0);
}

AppCordova.initialize();
