/**
* This component for popups/confirm/...
*/
var AppInterface = {
    hasModal: false,
    /// display a warning to go further
    alert: function(message, cbFunction, title, button){
        // replace callback by nothing
        if(cbFunction === undefined) {
            callback = function() { AppInterface.hasModal = false; };
        } else {
            callback = function() { AppInterface.hasModal = false; cbFunction(); };
        }
        // replace title by app Name
        if(title === undefined) {
            title = AppLocalization.getText("alertTitle");
            if(title === undefined) {
                title = AppCordova.getName();
            }
        }
        // call alert dialog
        if(AppInterface.hasModal == false) {
            AppInterface.hasModal = true;
            navigator.notification.alert(message, callback, title, button);
        } else {
             debug("AppInterface: cannot prompt alert");
        }
    },
    /// display a confirmation message to go further or no
    confirm: function(message, cbFunction, title, button){
        // replace callback by nothing
        if(cbFunction === undefined) {
            callback = function(buttonIndex) { AppInterface.hasModal = false; };
        } else {
            callback = function(buttonIndex) { AppInterface.hasModal = false; if(parseInt(buttonIndex) === 1) { cbFunction(); } };
        }
        // replace title if undefined
        if(title === undefined) {
            title = AppLocalization.getText("confirmTitle");
            if(title === undefined) {
                title = AppCordova.getName();
            }
        }
        // replace button if undefined
        if(button === undefined) {
            button = [AppLocalization.getText("yesButton"), AppLocalization.getText("noButton")];
        }
        // call confirm dialog
        if(AppInterface.hasModal == false) {
            AppInterface.hasModal = true;
            navigator.notification.confirm(message, callback, title, button);
        } else {
            debug("AppInterface: cannot prompt confirm");
        }
    },
    /// display a confirmation message to go further or no
    prompt: function(message, cbFunction, title, button, defaultText){
        // replace callback by nothing
        if(cbFunction === undefined) {
            callback = function(results) { AppInterface.hasModal = false; };
        } else {
            callback = function(results) { AppInterface.hasModal = false; cbFunction(parseInt(results.buttonIndex)-1, results.input1); };
        }
        // replace title if undefined
        if(title === undefined) {
            title = AppCordova.getName();
        }
        // replace button if undefined
        if(button === undefined) {
            button = ["Ok"];
        }
        // replace button if undefined
        if(defaultText === undefined) {
            defaultText = "";
        }
        // call confirm dialog
        if(AppInterface.hasModal == false) {
            AppInterface.hasModal = true;
            navigator.notification.prompt(message, callback, title, button, defaultText);
        } else {
            debug("AppInterface: cannot prompt");
        }
    },
    // play notification sound
    beep: function(times) {
        navigator.notification.beep(times);
    },
    /// display toast
    error: function(text, timeout) {
        if(text === undefined) {
            text = 'missingText';
        }
        window.plugins.toast.hide();
        if(timeout === undefined) {
            // infinite error
            window.plugins.toast.showWithOptions(
                { message: text, duration: 3600000, position: "center" },
                function(result) {
                    if (result && result.event) {
                        AppInterface.error(text, timeout);
                    }
                }
            );
        } else {
            // timeout error
            window.plugins.toast.show(text, timeout, 'center');
        }
    },
    success: function(text, timeout) {
        if(text === undefined) {
            text = 'missingText';
        }
        if(timeout === undefined) {
            timeout = 'long';
        }
        window.plugins.toast.hide();
        window.plugins.toast.show(text, 'long', 'center');
    }
};
