/**
/* This component to initialize languages and howrse version
**/
var AppLocalization = {
    lang: null,
    version: null,
    defaultLanguage: 'en_US',
    urlSupported: {
        fr_FR: {default: "m.ouranos.equideow.com", ouranos: "m.ouranos.equideow.com", gaia: "m.gaia.equideow.com"},
        en_US: {default: "m.howrse.com", en_US: "m.us.howrse.com", en_CA: "m.ca.howrse.com", en_GB: "m.howrse.co.uk", en_UK: "m.howrse.co.uk", en_AU: "m.au.howrse.com"},
        de_DE: {default: "m.howrse.de"},
        es_ES: {default: "m.caballow.com"},
        ru_RU: {default: "m.lowadi.com"},
        pl_PL: {default: "m.howrse.pl"},
        cs_CZ: {default: "m.howrse.cz"},
        no_NO: {default: "m.howrse.no"},
        nl_NL: {default: "m.nl.howrse.com"},
        da_DK: {default: "m.howrse.dk"},
        sv_SE: {default: "m.howrse.se"},
        fi_FI: {default: "m.howrse.fi"},
        hu_HU: {default: "m.howrse.hu"},
        ro_RO: {default: "m.howrse.ro"},
        sl_SI: {default: "m.howrse.si"},
        sk_SK: {default: "m.howrse.sk"},
        bg_BG: {default: "m.howrse.bg"},
        pt_BR: {default: "m.br.howrse.com"},
        pt_PT: {default: "m.howrse.com.pt"},
        it_IT: {default: "m.howrse.it"}
    },
    /// initialise language
    init: function(callback) {

        // default from memory
        this.lang = AppConfig.get('lang');
        this.version = AppConfig.get('version');
        // language
        if(this.lang != null) {
            // defined language
            this._setLang(this.lang, this.version);
            callback();
        } else {
            // retrieve from device
            navigator.globalization.getLocaleName(
                function(locale) {
                    // language returned as us-US
                    debug("[LOCALISATION] Current language : " + locale.value.replace('-', '_'));
                    if(BuildInfo.debug == true || BuildInfo.buildType === "debug") {
                        var language = "fr_FR";
                        debug("[LOCALISATION] Current language overrided by " + language);
                    } else {
                        var language = locale.value.replace('-', '_');
                    }
                    AppLocalization._setLang(language, null);
                    callback();
                },
                function() {
                    // if failed use default
                    AppLocalization._setLang(AppLocalization.defaultLanguage, null);
                    callback();
                }
            );
        }
    },
    /// set language
    _setLang: function(lang, version) {
        // case en_GB, en_CA, en_AU
        if(['en_UK', 'en_GB', 'en_CA', 'en_AU'].indexOf(lang) !== -1) {
            lang = AppLocalization.defaultLanguage;
        }
        // overwrite language is not supported
        if(this.urlSupported[this.lang] == undefined){
            // we try with only the lang, not the regional setting
            var prefixLang = lang.substr(0,2);
            for(key in this.urlSupported) {
                if(key.substr(0,2) == prefixLang) {
                    lang = key;
                    version = null;
                    break;
                }
            }
            if(this.urlSupported[lang] == undefined) {
                lang = this.defaultLanguage;
                version = null;
            }
        }
        // save language and return
        this.lang = lang;
        this.version = version;
        // get url
        var _url = this.getUrl();
        if(_url === undefined) {
            this._setLang(this.defaultLanguage, null);
        } else {
            // save settings
            AppConfig.set('lang', lang);
            AppConfig.set('version', version);
            // set document meta tags
            document.getElementsByTagName('html')[0].setAttribute('lang', lang.substr(0,2));
            // add base
            var base = document.createElement('base');
            base.href = _url + "/";
            document.getElementsByTagName('head')[0].appendChild(base);
        }
    },
    /// get language
    getLang: function() {
        return this.lang;
    },
    /// get version
    getVersion: function() {
        return this.version;
    },
    /// get url related to choosen language
    getUrl: function() {
        var url = this.urlSupported[this.defaultLanguage]['default'];
        if(this.urlSupported[this.lang] !== undefined) {
            var urls = this.urlSupported[this.lang];
            if(this.version !== null && urls[this.version] !== undefined) {
                url = urls[this.version];
            } else {
                url = urls['default'];
            }
        }
        // dev/prod
        if(BuildInfo.debug == true || BuildInfo.buildType === "debug") {
            url = url.replace('.equideow.', '.preprod.equideow.');
            url = url.replace('.howrse.', '.preprod.howrse.');
            url = url.replace('.caballow.', '.preprod.caballow.');
            url = url.replace('.lowadi.', '.preprod.lowadi.');
        }
        return "https://" + url;
    },
    /// change language on user input
    change: function(version) {
        switch(version) {
            //ouranos/gaia
            case 'ouranos' : case 'gaia' :
                this.lang='fr_FR';
                this.version=version;
                break;
            //us/ca/au/uk
            case 'en_US' : case 'en_CA' : case 'en_AU' : case 'en_GB' : case 'en_UK' :
                this.lang=this.defaultLanguage;
                this.version=version;
                break;
            // others
            default :
                if(this.urlSupported[version] != undefined) {
                    this.lang=version;
                    this.version=null;
                } else {
                    this.lang=this.defaultLanguage;
                    this.version=null;
                }
        }
        this._setLang(this.lang, this.version);
        AppCordova.reset();
        return true;
    },
    /// get text defined for each language
    getText: function(qName) {
        // get default language if none found
        if(eval(this.lang) !== null) {
            var text = this.lang;
        } else {
            var text = this.defaultLanguage;
        }
        // display error if no text found
        if(eval(text+"['"+qName+"']")) {
            return eval(text+"['"+qName+"']");
        } else {
            return "MissingText " + qName;
        }
    }
};
