/**
* This component to handle communication with server
*/
var AppRequest = {
    call: function(url, params, success, failure) {
        //network access
        if(AppCordova.checkNetwork() === false) {
            failure({errorCode: 1, message: AppLocalization.getText('networkNoneText')});
            return;
        }
        if(success === undefined) {
            success = function(o) { AppRequest.onSuccess(o); };
        }
        if(failure === undefined) {
            failure = function(o) { AppRequest.onFailure(o); };
        }
        this._cbCall(url, params, success, failure);
    },
    _cbCall: function(url, params, success, failure) {

        if(AppConfig.useAdvancedHttp == true) {
            return this.cbCallWithAdvancedHttp(url, params, success, failure);
        }

        return this.cbCallWithoutAdvancedHttp(url, params, success, failure);

    },
    cbCallWithoutAdvancedHttp: function(url, params, success, failure) {
        debug("[AppRequest] cbCallWithoutAdvancedHttp");
        debug("[AppRequest] " + params);
        var xhr = new XMLHttpRequest();
        xhr.open("POST", url, true);
        xhr.withCredentials = true;
        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        xhr.setRequestHeader("Cache-Control", "no-cache");
        xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
        xhr.onreadystatechange = function() {
            if(this.readyState == 4) {
                var status = parseInt(this.status);
                debug("AppRequest: "+this.status+" response");
                var response = undefined;
                try {
                    response = JSON.parse(this.responseText);
                } catch(err) {
                    debug("AppRequest: error parsing response");
                    return failure({errorCode: 2, message: AppLocalization.getText('internalText')});
                }

                if(status === 200) {
                    return success(response);
                }

                return failure(response);
            }
        }
        xhr.send(params);
    },
    cbCallWithAdvancedHttp: function(url, params, success, failure) {
        debug("[AppRequest] cbCallWithAdvancedHttp");

        cordova.plugin.http.setDataSerializer('urlencoded');

        debug("[AppRequest] Params : " + JSON.stringify(params, null, 4));

        const options = {
            method: 'post',
            data: params,
            headers : { CacheControl : "no-cache", XRequestedWith : "XMLHttpRequest" }
        }

        cordova.plugin.http.sendRequest(url, options, function(response) {
            return success(response);
        }, function(response) {
            return failure(response);
        });
    },
    onSuccess: function(o) {
        debug(o);
    },
    onFailure: function(o) {
        debug(o);
    }
}
